/*
 * SFUNMEM   A one integration-step delay and hold "memory" function.
 *           (see manual under Advanced Topics).
 *
 *           Syntax:  [sys, x0] = sfunmem(t,x,u,flag)
 *
 * This file designed to be used in a SIMULINK S-Function block.
 * It performs a one integration-step delay and hold "memory" function.
 * Thus, no matter how large or small the last time increment was in
 * the integration process, this function will hold the input variable
 * from the last integration step.
 *
 * Use this function with a clock as input to get the step-size at each
 * step of a simulation.
 *
 * Andrew Grace & Ned Gulley  April 3, 1992
 * Copyright (c) 1993-94 The MathWorks, Inc.
 * All Rights Reserved
 */

/*
 * The following #define is used to specify the name of this S-Function.
 */

#define S_FUNCTION_NAME sfunmem

/*
 * need to include simstruc.h for the definition of the SimStruct and
 * its associated macro definitions.
 */

#include "simstruc.h"

/*
 * mdlInitializeSizes - initialize the sizes array
 *
 * The sizes array is used by SIMULINK to determine the S-function block's
 * characteristics (number of inputs, outputs, states, etc.).
 */

static void mdlInitializeSizes(S)
    SimStruct *S;
{
    ssSetNumContStates(    S, 0);                  /* number of continuous states */
    ssSetNumDiscStates(    S, 0);                  /* number of discrete states */
    ssSetNumInputs(        S, DYNAMICALLY_SIZED);  /* number of inputs */
    ssSetNumOutputs(       S, DYNAMICALLY_SIZED);  /* number of outputs */
    ssSetDirectFeedThrough(S, 0);                  /* direct feedthrough flag */
    ssSetNumSampleTimes(   S, 1);                  /* number of sample times */
    ssSetNumInputArgs(     S, 0);                  /* number of input arguments */
    ssSetNumRWork(         S, DYNAMICALLY_SIZED);  /* number of real work vector elements */
    ssSetNumIWork(         S, 0);                  /* number of integer work vector elements */
    ssSetNumPWork(         S, 0);                  /* number of pointer work vector elements */
}

/*
 * mdlInitializeSampleTimes - initialize the sample times array
 *
 * This function is used to specify the sample time(s) for your S-function.
 * If your S-function is continuous, you must specify a sample time of 0.0.
 * Sample times must be registered in ascending order.
 */

static void mdlInitializeSampleTimes(S)
    SimStruct *S;
{
    ssSetSampleTimeEvent(S, 0, 0.0);
    ssSetOffsetTimeEvent(S, 0, 0.0);
}

/*
 * mdlInitializeConditions - initialize the states
 *
 * In this function, you should initialize the continuous and discrete
 * states for your S-function block.  The initial states are placed
 * in the x0 variable.  You can also perform any other initialization
 * activities that your S-function may require.
 */

static void mdlInitializeConditions(x0, S)
    double *x0;
    SimStruct *S;
{
    double *RWork = ssGetRWork(S);
    int i;
    
    /*
     * the memory state is stored in the RWork vector, initialize it
     * to zero
     */

    for (i = 0; i < ssGetNumRWork(S); i++)
	RWork[i] = 0.0;
}

/*
 * mdlOutputs - compute the outputs
 *
 * In this function, you compute the outputs of your S-function
 * block.  The outputs are placed in the y variable.
 */

static void mdlOutputs(y, x, u, S, tid)
    double *y, *x, *u;
    SimStruct *S;
    int tid;
{
    double *RWork = ssGetRWork(S);
    int i;
    
    /*
     * Output the last input, which is stored in the real work vector
     */

    for (i = 0; i < ssGetNumRWork(S); i++)
	y[i] = RWork[i];
}

/*
 * mdlUpdate - perform action at major integration time step
 *
 * This function is called once for every major integration time step.
 * Discrete states are typically updated here, but this function is useful
 * for performing any tasks that should only take place once per integration
 * step.
 */

static void mdlUpdate(x, u, S, tid)
    double *x, *u;
    SimStruct *S;
    int tid;
{
    double *RWork = ssGetRWork(S);
    int i;
    
    /*
     * Just set memory to last input
     */

    for (i = 0; i < ssGetNumRWork(S); i++)
	RWork[i] = u[i];
}

/*
 * mdlDerivatives - compute the derivatives
 *
 * In this function, you compute the S-function block's derivatives.
 * The derivatives are placed in the dx variable.
 */

static void mdlDerivatives(dx, x, u, S, tid)
    double *dx, *x, *u;
    SimStruct *S;
    int tid;
{
}

/*
 * mdlTerminate - called when the simulation is terminated.
 *
 * In this function, you should perform any actions that are necessary
 * at the termination of a simulation.  For example, if memory was allocated
 * in mdlInitializeConditions, this is the place to free it.
 */

static void mdlTerminate(S)
    SimStruct *S;
{
}

#ifdef	MATLAB_MEX_FILE    /* Is this file being compiled as a MEX-file? */
#include "simulink.c"      /* MEX-file interface mechanism */
#else
#include "cg_sfun.h"       /* Code generation registration function */
#endif
